require 'java'

include_class "java.lang.Runnable"
include_class "javax.swing.JTextArea"
include_class "java.awt.Font"
include_class "javax.swing.text.Segment"
include_class "javax.swing.SwingUtilities"
include_class "jruby.tools.ConsoleUI"

class JTextComponentConsoleUI < ConsoleUI

  DEFAULTCOLUMNS = 80  
  DEFAULTROWS = 24
   
  attr_accessor(:model, :textComponent, :mark)
  
  def initialize()
    super()
    @textComponent = createTextComponent()
    @mark = 0 #?
  end
  
  def createTextComponent()
    textarea = JTextArea.new()
    textarea.setLineWrap(true)
    textarea.setFont(Font.new("Monospaced", 0, 12))
    textarea.setRows(DEFAULTROWS)
    textarea.setColumns(DEFAULTCOLUMNS)
    textarea
  end
  
  def runCommand(command)
    if (!SwingUtilities.isEventDispatchThread())
      SwingUtilities.invokeLater(command)
    else
      command.run()
    end
  end
  
  def append(text)
    runCommand(Command.new(self, text))
  end
  
  def insert(str, mark)
    doc = @textComponent.getDocument()
    if (doc != nil)
      begin
        doc.insertString(mark, str, nil)
      rescue BadLocationException => e
        raise IllegalArgumentException(e.getMessage())
      end
    end
  end
  
  def getLength()
    @textComponent.getDocument().getLength()
  end
  
  def setCursorPosition(pos)
    textComponent.select(pos, pos)
  end
    
  def enter()
    begin
      len = getLength()
      sendText(@mark, len - @mark)
    rescue IOException => e
      System.err.println(e)
    end
  end
  
  def entered(command)
    # skip
  end
  
  def sendText(start, len)
    doc = textComponent.getDocument()
    doclen = doc.getLength()
    segment = Segment.new()
    begin
      doc.getText(@mark, doclen - @mark, segment)
    rescue BadLocationException => ble
      ble.printStackTrace()
    end
    entered(segment.toString())
    cbuf = segment.array
    offset = segment.offset
    count = segment.count
    
    @model.enter(cbuf, offset, count)
    
    insert("\n", getLength())
    @mark = doc.getLength()
  end
  
  def close()
  end
end

class Command < Runnable
  
  def initialize(jtextcomponentcosoleui,str)
    super()
    @jtextcomponentcosoleui = jtextcomponentcosoleui
    @str = str
  end
  
  def run()
    @jtextcomponentcosoleui.insert(@str, @jtextcomponentcosoleui.mark)
    @jtextcomponentcosoleui.mark += @str.length()
    pos = @jtextcomponentcosoleui.getLength()
    @jtextcomponentcosoleui.setCursorPosition(pos)
  end
end
