/*
 * @(#)Console.java 1.4 05/05/16
 *
 * Copyright (c) 1997-2005 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "LICENSE.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package jruby.tools;

import java.io.IOException;
import java.io.Writer;

public class ConsoleBuffer extends Writer {
	private StringBuffer buf = new StringBuffer();

	private int lineBufferSize;

	private int pendingLines = 0;

	private ConsoleUI ui;

	public ConsoleBuffer(int lineBufferSize) {
		this.lineBufferSize = lineBufferSize;
	}

	public synchronized void write(int ch) {
		buf.append((char) ch);
		if (lineBufferSize > 0 && ch == '\n') {
			if (++pendingLines > lineBufferSize) {
				flushBuffer();
				pendingLines = 0;
			}
		}
	}

	public synchronized void write(char[] data, int off, int len) {
		for (int i = off; i < len; i++) {
			buf.append(data[i]);
			if (lineBufferSize > 0 && data[i] == '\n') {
				if (++pendingLines > lineBufferSize) {
					flushBuffer();
					pendingLines = 0;
				}
			}
		}
	}

	public synchronized void flush() throws IOException {
		if (buf.length() > 0) {
			flushBuffer();
		}
	}

	public void close() throws IOException {
		flush();
	}

	public int size() {
		return buf.length();
	}

	private void flushBuffer() {
		final String str = buf.toString();
		buf.setLength(0);
		if (ui != null) {
			ui.append(str);
		}
	}

	public void setUI(ConsoleUI ui) {
		this.ui = ui;
	}
}
