require 'java'

require 'JTextComponentConsoleUI'

include_class "java.awt.event.KeyEvent"
include_class ["KeyListener"].map {|name| "java.awt.event." + name}
include_class "java.awt.event.WindowListener"
include_class "javax.swing.JFrame"
include_class "javax.swing.JScrollPane"
include_class "java.awt.Toolkit"
include_class "java.lang.ClassLoader"
include_class "jruby.tools.DocumentAndKeyListener"


def getResource(resourcename)  
  url=ClassLoader.getSystemResource(resourcename)
end

class FrameWindowListener < WindowListener
  
  def initialize(model)
    super()
    @model = model
  end
  
  def windowClosing(e)
    @model.dispose
  end
  
  def windowActivated(e)
  end
  
  def windowOpened(e)
  end
  
  def windowDeactivated(e)
  end
  
  def windowIconified(e)
  end
  
  def windowDeiconified(e)
  end
end

# Swing based UI
class JRubyConsoleUI < JTextComponentConsoleUI
  
  attr_accessor(:title, :frame, :component)
  
  ICON_RESOURCE = "ruby16.png"  
  
  def initialize(console = nil, title ="", binding = nil)
    super()    
    @model = console
    @handler = EventHandler.new(self)
    textarea = @textComponent
    textarea.addKeyListener(@handler)
    textarea.getDocument().addDocumentListener(@handler)
    #
    @title = title
    @binding = binding
    console = createConsole5(nil, nil, true, nil,JThread::NORM_PRIORITY)
    getFrame().setVisible(true)    
    console.start()
  end
  
  def createConsole1(inputlog)
    createConsole(nil, inputlog, true, nil, Thread.NORM_PRIORITY)
  end
  
  #protected
  def createConsole()
    JRubyConsole.new(@binding)
  end
  
  def createConsole5(cl, inputlog, greeting, terminationCallback, priority)
    console = createConsole()
    @model = console
    console.inputLog = inputlog
    console.greeting = greeting
    console.classloader = cl
    console.priority = priority
    console.terminationCallback = terminationCallback
    console.ui = self
    console 
  end
  
  def entered(command)
    @handler.entered(command)
  end
  
  def getTextArea()
    @jrubyconsoleui.textComponent
  end
  
  def getComponent()
    if (@component == nil)
      @component = JScrollPane.new(@textComponent)
    end
    @component
  end  
  
  def getFrame()
    if (@frame == nil)
      @frame = JFrame.new(title)
      @frame.setContentPane(getComponent())
      @frame.setSize(@textComponent.getPreferredSize())
      imageURL = getResource(ICON_RESOURCE)
      iconImage = Toolkit.getDefaultToolkit().getImage(imageURL)     
      @frame.setIconImage(iconImage)      
      @frame.addWindowListener(FrameWindowListener.new(@model))
      return @frame
    end
  end
  
  #private
  def largerFont()
    font = @jrubyconsoleui.textComponent.getFont()
    size = font.getSize() + 1
    @jrubyconsoleui.textComponent.setFont(font.deriveFont(size))
  end
  
  def smallerFont()
    font = @jrubyconsoleui.textComponent.getFont()
    int size = font.getSize() - 1
    if (size < 1)
      size = 1
    end
    @jrubyconsoleui.textComponent.setFont(font.deriveFont(size))
  end
  
  def close()
    if (frame != nil)
      frame.dispose()
    end
  end
  
end

class EventHandler < DocumentAndKeyListener
  
  def initialize(jrubyconsoleui)
    super()
    @jrubyconsoleui = jrubyconsoleui
    @history = []
    @historyIndex = -1
  end
  
  def entered(command)    
    if (command != nil && command.length() > 0)
      @history << command
    end
    @historyIndex = @history.size
  end  
  
  def keyPressed(e)
    code = e.getKeyCode()
    ch = e.getKeyChar()
    control = e.isControlDown()
    
    pos = @jrubyconsoleui.textComponent.getCaretPosition()
    if (code == KeyEvent::VK_BACK_SPACE || code == KeyEvent::VK_LEFT)
      if (@jrubyconsoleui.mark == pos)
        e.consume()
      end            
    elsif (control && code == KeyEvent::VK_A)
      @jrubyconsoleui.textComponent.setCaretPosition(@jrubyconsoleui.mark)
      e.consume()
    elsif (control && code == KeyEvent::VK_E)
      @jrubyconsoleui.textComponent.setCaretPosition(@jrubyconsoleui.textComponent.getDocument().getLength())
      e.consume()
    elsif (control && code == KeyEvent::VK_B)
      if (@jrubyconsoleui.mark < pos)
        @jrubyconsoleui.textComponent.setCaretPosition(pos - 1)
      end
      e.consume()
    elsif (control && code == KeyEvent::VK_D)
      doc = @jrubyconsoleui.textComponent.getDocument()
      if (pos < doc.getLength())
        begin
          doc.remove(pos, 1)
        rescue BadLocationException => ble
          ble.printStackTrace()
        end
      end
      e.consume()
    elsif (control && code == KeyEvent::VK_F)
      if (@jrubyconsoleui.textComponent.getDocument().getLength() > pos)
        @jrubyconsoleui.textComponent.setCaretPosition(pos + 1)
      end
      e.consume()
    elsif (control && code == KeyEvent::VK_K)
      doc = @jrubyconsoleui.textComponent.getDocument()
      begin
        doc.remove(pos, doc.getLength() - pos)
      rescue BadLocationException => ble
        ble.printStackTrace()
      end
      e.consume()
    elsif (control && code == KeyEvent::VK_U)
      doc = @jrubyconsoleui.textComponent.getDocument()
      begin
        doc.remove(@jrubyconsoleui.mark, doc.getLength() - @jrubyconsoleui.mark)
      rescue BadLocationException => ble
        ble.printStackTrace()
      end
      e.consume()
    elsif (control && ch == '+')
      largerFont()
      e.consume()
    elsif (control && (ch == '-' || code == KeyEvent::VK_MINUS))
      smallerFont()
      e.consume()
    elsif (code == KeyEvent::VK_HOME || (control && code == KeyEvent::VK_A))
      if (pos == @jrubyconsoleui.mark)
        e.consume()
      elsif (pos > @jrubyconsoleui.mark) 
        if (!control)
          if (e.isShiftDown())
            @jrubyconsoleui.textComponent.moveCaretPosition(@jrubyconsoleui.mark)
          else
            @jrubyconsoleui.textComponent.setCaretPosition(@jrubyconsoleui.mark)
          end
          e.consume()
        end
      end
    elsif (code == KeyEvent::VK_ENTER)
      @jrubyconsoleui.enter()
      e.consume()
    elsif (code == KeyEvent::VK_UP || (control && code == KeyEvent::VK_P))
      @historyIndex -= 1
      if (@historyIndex >= 0)
        if (@historyIndex >= @history.size())
          @historyIndex = @history.size() -1
        end
        if (@historyIndex >= 0)
          str = @history[@historyIndex]
          len = @jrubyconsoleui.textComponent.getDocument().getLength()
          @jrubyconsoleui.textComponent.replaceRange(str, @jrubyconsoleui.mark, len)
          caretPos = @jrubyconsoleui.mark + str.length()
          @jrubyconsoleui.textComponent.select(caretPos, caretPos)
        else
          @historyIndex+=1
        end
      else
        @historyIndex+=1
      end
      e.consume()
    elsif (code == KeyEvent::VK_DOWN || (control && code == KeyEvent::VK_N))
      caretPos = @jrubyconsoleui.mark
      if (@history.size > 0)
        @historyIndex+=1
        if (@historyIndex < 0)
          @historyIndex = 0
        end
        len = @jrubyconsoleui.textComponent.getDocument().getLength()
        if (@historyIndex < @history.size)
          str = @history[@historyIndex]
          @jrubyconsoleui.textComponent.replaceRange(str, @jrubyconsoleui.mark, len)
          caretPos = @jrubyconsoleui.mark + str.length()
        else
          @historyIndex = @history.size()
          @jrubyconsoleui.textComponent.replaceRange("", @jrubyconsoleui.mark, len)
        end
      end
      @jrubyconsoleui.textComponent.select(caretPos, caretPos)
      e.consume()
      
    end
  end
  
  def keyTyped(e)
    keyChar = e.getKeyChar()
    if (keyChar == KeyEvent::VK_BACK_SPACE)      
      if (@jrubyconsoleui.mark == @jrubyconsoleui.textComponent.getCaretPosition())
        e.consume()
      end
    elsif (@jrubyconsoleui.textComponent.getCaretPosition() < @jrubyconsoleui.mark)
      @jrubyconsoleui.textComponent.setCaretPosition(@jrubyconsoleui.mark)
    end
  end
  
  def keyReleased(e)
  end
  
  def insertUpdate(e)
    #synchronized (JRubyConsoleUI.this){
    len = e.getLength()
    off = e.getOffset()   
    if (@jrubyconsoleui.mark > off)
      @jrubyconsoleui.mark += len
    end
    #end synchronized
  end
  
  def removeUpdate(e)
    #synchronized (JRubyConsoleUI.this){
    len = e.getLength()
    off = e.getOffset()
    if (@jrubyconsoleui.mark > off)
      if (@jrubyconsoleui.mark >= off + len)
        @jrubyconsoleui.mark -= len
      else
        @jrubyconsoleui.mark = off
      end
    end
    #end synchronized
  end
  
  def changedUpdate(e)
  end
end
