require 'java'

require 'console'
require 'jrubyconsoleui'

include_class "java.lang.Runnable"
include_class "java.lang.System"
include_class "java.lang.Throwable"
include_class "java.lang.StringBuffer"
include_class("java.lang.Thread") {|package, name| "J#{name}"}
include_class "java.io.PrintWriter"
include_class "java.io.BufferedReader"
include_class "java.io.IOException"
include_class "java.io.StringReader"
include_class "org.jruby.runtime.Constants"

class MyRunnable < Runnable
  
  def initialize(r,w, jrubyconsoleui)
    super()
    @java_vendor = System.getProperty("java.vendor")
    @java_version = System.getProperty("java.version")
    @showgreeting = true
    @r = r
    @w = w
    @jrubyconsoleui = jrubyconsoleui    
  end
  
  def version()
    Constants::VERSION + " (Ruby " + Constants::RUBY_VERSION + ")"
  end
  
  def greeting(writer)    
    w = PrintWriter.new(writer)
    w.print("JRuby version " + version() + ", " + @java_version + + " ("+ @java_vendor + ")")
    vm_name = System.getProperty("java.vm.name")
    vm_info = System.getProperty("java.vm.info")
    if (vm_name != nil)
      w.print("\n" + vm_name)
      if (vm_info != nil)
        w.println(" (" + vm_info + ")")
      else
        w.println()
      end
    else
      w.println()
    end
  end
  
  def run()
    if (@showgreeting)    
      greeting(@w)
    end
    @jrubyconsoleui.load(@r, @w)
    close()
    begin
      @w.close()
    rescue IOException => e
      # ignore
    end
    begin
      @r.close()
    rescue IOException => e
      # ignore
    end
  end
end


class JRubyConsole < Console
  
  PROMPT = "> "
  
  THREAD_NAME = "Pnuts Console"
  
  GREETING = "pnuts.tools.PnutsConsole.greeting"
  
  INPUTLOG = "pnuts.tools.PnutsConsole.inputlog"
  
  attr_accessor(:modules, :greeting, :classloader, :inputLog)
  attr_accessor(:priority, :terminationCallback)
  
  def initialize(b)
    super()
    @binding = b
    @priority = JThread::NORM_PRIORITY
  end
  
  def start()
    if (@ui == nil)
      raise IllegalStateException("no UI ")
    end
    w = @out
    
    #// c.setWriter(w);
    #// c.setErrorWriter(w);
    #// c.setTerminalWriter(w);
    r = getReader()
    
    
    @th = JThread.new( MyRunnable.new(r,w,self), THREAD_NAME)
    @th.setDaemon(true)
    @th.setPriority(priority)
    if (@classloader != nil)
      @th.setContextClassLoader(classLoader)
    end
    
    @th.start()
  end
  
  def close()
    if (terminationCallback != nil)
      terminationCallback.run()
    end
    ui.close()
  end
  
  def dispose()
    #enter("quit()\n")
    @ini.close()
    @out.close()
    if (@th != nil)
      @th.interrupt()
    end
  end
  
  #public synchronized Reader 
  def getReader()
    if (@ini == nil)
      if (@inputlog != nil)
        begin
          @ini = LogReader.new(super.getReader(), inputlog)
        rescue IOException => e
          @ini = super.getReader()
        end
      else 
        @ini = super.getReader()
      end
    end
    @ini
  end
  
  def load(reader, term)
    session(reader, term)
  end
  
  def session(r, writer)
    br = BufferedReader.new(r)
    sbuf = StringBuffer.new()
    value = nil
    term = PrintWriter.new(writer)
    while (true)
      begin
        if (term != nil)
          term.print(PROMPT)
          term.flush()
        end
        line = br.readLine()
        if (line == nil)
          value
        end
        sbuf.append(line)
        sr = StringReader.new(sbuf.toString())
        expr = sbuf.toString()
        value = eval(line, @binding)
        term.println(value.to_s)
        term.flush()      
      rescue => e
        term.println(e.message)#printStackTrace(term)
        #e.printBacktrace(term) - no sei imprimir stacktraces em condies
      end
    end
  end
end

def console(title ="")
  JRubyConsoleUI.new(nil, title, binding)
end

#console("JRuby Swing Console")
